import Logger from 'nightingale';
import { hostname } from 'os';
import createSocketClient from 'socket.io-client';
import { serverHost, serverPort, userId } from './argv';
import { getTime as getConfigTime, updateConfig } from './config';
import * as display from './display';
import findNetworkInterface from './networkInterface';
import * as screen from './screen';
import { selfUpdate } from './update';
import { version } from '../package.json';

const logger = new Logger('app:client');

const socket = serverHost && serverPort && (
  createSocketClient(`${serverHost}:${serverPort}/raspberry-client`, {
    reconnectionDelay: 500,
    reconnectionDelayMax: 120000,
    timeout: 4000,
    transports: ['websocket'],
  })
);

process.nextTick(() => {
  logger.debug('Connecting', { serverHost, serverPort });
  if (socket) {
    socket.connect();
  }
});

const rpiVersion = (() => {
  try {
    // eslint-disable-next-line
    return require('raspi-ver');
  } catch (err) {
    return null;
  }
})();


function emit(eventName: string, ...args) {
  if (!socket) {
    logger.warn('cannot emit', { eventName, args });
    return;
  }
  logger.debug('emit', { eventName, args });
  return socket.emit(eventName, ...args);
}

if (socket) {
  socket.on('connect_error', err => logger.error('connect error', { serverHost, serverPort, errMessage: err.message }));
  socket.on('reconnect_error', err => logger.debug('reconnect error', { serverHost, serverPort, err }));

  socket.on('disconnect', () => logger.warn('disconnected'));

  socket.on('reconnect', () => logger.success('reconnected'));

  socket.on('connect', () => {
    logger.success('connected');

    const networkInterface = findNetworkInterface();
    emit('hello', {
      configTime: getConfigTime(),
      version,
      userId,
      screenState: screen.getCurrentScreenState(),
      ...networkInterface,
      hostname: hostname(),
      rpiVersion,
    });
  });

  socket.on('updateConfig', (config) => {
    if (updateConfig(config)) {
      display.update();
    }
  });

  socket.on('changeConfig', (config) => {
    if (updateConfig(config)) {
      display.update();
    }
  });

  socket.on('selfUpdate', () => selfUpdate());

  socket.on('action', (action: string) => {
    logger.info('received action', { action });
    switch (action) {
      case 'self-upgrade':
      case 'self-update':
      case 'selfUpdate':
        return selfUpdate();

      case 'screen-off':
        return screen.off();
      case 'screen-on':
        return screen.on();

      case 'refresh':
        return display.refresh();
    }

    logger.warn(`unknown action: ${action}`);
  });

  socket.on('screenshot', () => {
    logger.info('screenshot');
    const buffer = screen.screenshot();
    if (!buffer) {
      return;
    }

    logger.info('send screenshot');
    socket.emit('screenshot', { image: true, buffer }, () => {
      logger.info('screenshot callback');
    });
  });
}

export function sendUpdate(data: Object): void {
  emit('update', data);
}

export function close(): Promise<any> | void {
  if (!socket || !socket.connected) {
    return;
  }

  return new Promise((resolve) => {
    logger.info('Closing...');
    socket.once('disconnect', () => {
      logger.info('Closed');
      resolve();
    });
    socket.close();
  });
}
