/* eslint-disable no-use-before-define */
import Logger from 'nightingale';
import { piUser } from './argv';
import { get as getConfig } from './config';
import { runScript, spawn } from './scripts';

const [uid, gid] = ['u', 'g'].map((kind: string) => {
  const res = Number(runScript('id', [`-${kind}`, piUser]));
  if (!res) {
    console.error(`Missing ${kind}id for user pi`);
    process.exit(1);
  }

  return res;
});

const runScriptAsPi = (script: string, args: Array<string | number>) => (
  runScript(script, args, { uid, gid, env: { USER: piUser, HOME: `/home/${piUser}` } })
);

const spawnAsPi = (script: string, args: Array<string | number>) => (
  spawn(script, args, { uid, gid, env: { USER: piUser, HOME: `/home/${piUser}` } })
);

const logger = new Logger('app:display');

let currentDisplay;
let autoRestart;
let childProcess;

const displays = {
  chromium: { openbox: true },
  livestreamer: { openbox: false },
  omxplayer: { openbox: false },
};

export function refresh() {
  if (childProcess) {
    logger.log('refresh');
    const config = getConfig();
    return runScriptAsPi(`./${config.display}.sh`, ['refresh']);
  } else {
    logger.warn('display stopped');
  }
}

export function update() {
  logger.info('update');
  const config = getConfig();
  const displayOptions = displays[currentDisplay];
  if (currentDisplay === config.display && displayOptions && displayOptions.canLoad) {
    return runScriptAsPi(`./${config.display}.sh`, ['load', config.url]);
  } else {
    return restart();
  }
}

function startOpenBox() {
  logger.info('start openbox');
  return runScript('./openbox.sh', ['start']);
}

function stopOpenBox() {
  logger.info('stop openbox');
  return runScript('./openbox.sh', ['stop']);
}

function startChild() {
  logger.info('starting child');
  if (childProcess) {
    throw new Error('child process already started');
  }


  autoRestart = true;

  const config = getConfig();

  currentDisplay = config.display;
  logger.info('start', { display: currentDisplay, url: config.url });
  childProcess = spawnAsPi(`./${currentDisplay}.sh`, ['start', config.url]);
  const thisChildProcess = childProcess;
  childProcess.stdout.on('data', data => logger.debug(data.toString()));
  childProcess.stderr.on('data', data => logger.error(data.toString()));
  childProcess.on('close', (code) => {
    const sameChildProcess = thisChildProcess === childProcess;
    childProcess = null;
    logger.error(`child process exited with code ${code}`);
    if (autoRestart && sameChildProcess) {
      process.nextTick(() => start());
    }
  });
}

export function openboxStarted() {
  const config = getConfig();
  logger.info('openbox started', { display: config.display });

  if (!displays[config.display].openbox) {
    stopOpenBox();
  } else {
    startChild();
  }
}

export function start() {
  if (childProcess) {
    logger.warn('start: already started');
    return;
  }

  const config = getConfig();

  if (!displays[config.display]) {
    if (config.display !== 'none') {
      logger.warn('Not supported', { display: config.display });
    }
    return;
  }

  logger.info('starting...', { config });

  if (!displays[config.display].openbox) {
    stopOpenBox();
    startChild();
  } else if (startOpenBox() !== 'started') {
    logger.info('openbox not yet started');
  } else {
    startChild();
  }
}

export function restart() {
  logger.info('restarting...');
  stop();
  return start();
}

export function stop() {
  autoRestart = false;
  if (childProcess) {
    childProcess.removeAllListeners();
    childProcess.kill();
  }
  childProcess = null;

  logger.info('stop', { display: currentDisplay });
  runScriptAsPi('./display.sh', ['stop']);
  stopOpenBox();
}
